"use strict";
// *****************************************************************************
// Copyright (C) 2018 Ericsson and others.
//
// This program and the accompanying materials are made available under the
// terms of the Eclipse Public License v. 2.0 which is available at
// http://www.eclipse.org/legal/epl-2.0.
//
// This Source Code may also be made available under the following Secondary
// Licenses when the conditions for such availability set forth in the Eclipse
// Public License v. 2.0 are satisfied: GNU General Public License, version 2
// with the GNU Classpath Exception which is available at
// https://www.gnu.org/software/classpath/license.html.
//
// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
// *****************************************************************************
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchOnigasm = exports.createOnigasmLib = exports.dynamicOnigasmLib = exports.OnigasmLib = void 0;
const browser_1 = require("@theia/core/lib/browser");
const core_1 = require("@theia/core");
const textmate_registry_1 = require("./textmate-registry");
const textmate_contribution_1 = require("./textmate-contribution");
const monaco_textmate_service_1 = require("./monaco-textmate-service");
const monaco_theme_registry_1 = require("./monaco-theme-registry");
const onigasm_1 = require("onigasm");
class OnigasmLib {
    createOnigScanner(sources) {
        return new onigasm_1.OnigScanner(sources);
    }
    createOnigString(sources) {
        return new onigasm_1.OnigString(sources);
    }
}
exports.OnigasmLib = OnigasmLib;
exports.default = (bind, unbind, isBound, rebind) => {
    bind(monaco_textmate_service_1.OnigasmPromise).toDynamicValue(dynamicOnigasmLib).inSingletonScope();
    bind(monaco_textmate_service_1.MonacoTextmateService).toSelf().inSingletonScope();
    bind(browser_1.FrontendApplicationContribution).toService(monaco_textmate_service_1.MonacoTextmateService);
    (0, core_1.bindContributionProvider)(bind, textmate_contribution_1.LanguageGrammarDefinitionContribution);
    bind(textmate_registry_1.TextmateRegistry).toSelf().inSingletonScope();
    bind(monaco_theme_registry_1.MonacoThemeRegistry).toDynamicValue(() => monaco_theme_registry_1.MonacoThemeRegistry.SINGLETON).inSingletonScope();
};
async function dynamicOnigasmLib(ctx) {
    return createOnigasmLib();
}
exports.dynamicOnigasmLib = dynamicOnigasmLib;
async function createOnigasmLib() {
    if (!browser_1.isBasicWasmSupported) {
        throw new Error('wasm not supported');
    }
    const wasm = await fetchOnigasm();
    await (0, onigasm_1.loadWASM)(wasm);
    return new OnigasmLib();
}
exports.createOnigasmLib = createOnigasmLib;
async function fetchOnigasm() {
    // Using Webpack's wasm loader should give us a URL to fetch the resource from:
    const onigasmPath = require('onigasm/lib/onigasm.wasm');
    const response = await fetch(onigasmPath, { method: 'GET' });
    return response.arrayBuffer();
}
exports.fetchOnigasm = fetchOnigasm;
//# sourceMappingURL=monaco-textmate-frontend-bindings.js.map